@@ -315,6 +315,13 @@ setMethod("clamp", signature(x="SpatRaster"),
 	}
 )
 
+setMethod("clamp_ts", signature(x="SpatRaster"),
+	function(x, min=FALSE, max=TRUE, filename="", ...) {
+		opt <- spatOptions(filename, ...)
+		x@ptr <- x@ptr$clamp_ts(min, max, opt)
+		messages(x, "clamp_ts")
+	}
+)
 
 setMethod("clamp", signature(x="numeric"),
 function(x, lower=-Inf, upper=Inf, values=TRUE, ...) {
